﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows;
using System.Collections;
using System.Collections.ObjectModel;

namespace ListBoxToTileView
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			this.firstTileView.ItemsSource = GenerateList();

			firstTileView.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
			firstTileView.AddHandler(RadDragAndDropManager.DragInfoEvent, new EventHandler<DragDropEventArgs>(OnDragInfo));

			firstTileView.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
			firstTileView.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnDropInfo));

			this.firstTileView.Loaded += this.firstTileView_Loaded;
		}

		private void firstTileView_Loaded(object sender, RoutedEventArgs e)
		{
			var tileView = sender as RadTileView;
			if (tileView != null)
			{
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					for (int i = 0; i < tileView.Items.Count; i++)
					{
						var container = tileView.ItemContainerGenerator.ContainerFromIndex(i) as RadTileViewItem;
						if (container != null)
						{
							var innerTileView = container.FindChildByType<RadTileView>();
							if (innerTileView != null)
							{
								innerTileView.AddHandler(RadDragAndDropManager.DragQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDragQuery));
								innerTileView.AddHandler(RadDragAndDropManager.DragInfoEvent, new EventHandler<DragDropEventArgs>(OnDragInfo));

								innerTileView.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnDropQuery));
								innerTileView.AddHandler(RadDragAndDropManager.DropInfoEvent, new EventHandler<DragDropEventArgs>(OnDropInfo));
								innerTileView.Loaded += this.firstTileView_Loaded;
							}
						}
					}
				}));
			}
		}

		public ObservableCollection<DataItem> GenerateList()
		{
			ObservableCollection<DataItem> result = new ObservableCollection<DataItem>();
			for (int i = 0; i < 4; i++)
			{
				DataItem item = new DataItem();
				item.Header = String.Format("Header {0}", i);
				item.Description = String.Format("Description {0}", i);
				item.Details = String.Format("Details {0}", i);
				for (int j = 0; j < 4; j++)
				{
					item.InnerItems.Add(new DataItem()
					{
						Header = String.Format("InnerHeader {0}.{1}", i, j),
						Description = String.Format("InnerDescription {0}.{1}", i, j),
						Details = String.Format("InnerDetails {0}.{1}", i, j)
					});
				}


				result.Add(item);
			}
			return result;
		}

		// OnDragQuery event handler
		private void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			RadTileViewItem container = e.Options.Source as RadTileViewItem;

			//draggedItem = e.Options.Source;
			if (e.Options.Status == DragStatus.DragQuery && container != null)
			{
				e.QueryResult = true;
				e.Handled = true;
				// Create Drag and Arrow Cue
				ContentControl dragCue = new ContentControl();
				dragCue.Content = container.DataContext;
				dragCue.ContentTemplate = this.Resources["DragCueTemplate"] as DataTemplate;
				e.Options.DragCue = dragCue;
				e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
				// Set the payload (this is the item that is currently dragged)
				e.Options.Payload = container.ParentTileView.ItemContainerGenerator.ItemFromContainer(container);
			}
			if (e.Options.Status == DragStatus.DropSourceQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}

		// OnDropQuery event handler
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			e.Handled = true;
			e.QueryResult = true;
		}

		// OnDropInfo event handler
		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			RadTileView tileViewSource = null;
			if ((e.Options.Source as RadTileView) != null)
				tileViewSource = e.Options.Source as RadTileView;
			else
				tileViewSource = e.Options.Source.ParentOfType<RadTileView>();

			if (e.Options.Destination != tileViewSource)
			{
				System.Windows.Controls.ItemsControl destination = e.Options.Destination as System.Windows.Controls.ItemsControl;

				if (e.Options.Status == DragStatus.DropComplete && destination != null && sender == e.Options.Destination)
				{
					// Its not a good practice to cast the ItemsSource like this!
					// You should implement your custom logic here.
					(destination.ItemsSource as IList).Add(e.Options.Payload);
				}
			}
		}

		// OnDragInfo event handler
		private void OnDragInfo(object sender, DragDropEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragComplete && e.Options.Destination != sender)
			{
				//remove the dragged item from the ItemsControl
				System.Windows.Controls.ItemsControl source = sender as System.Windows.Controls.ItemsControl;

				// Its not a good practice to cast the ItemsSource like this!
				// You should implement your custom logic here.
				(source.ItemsSource as IList).Remove(e.Options.Payload);
			}
		}
	}

}
